package CodeGen;

/**
 * 
 * @author Galia
 * For each remote procedure call,there needs to be a way to 
 * generate code that can create and parse the RCP. 
 */
public interface IMethodGen {	
	/**
	 * It is most likely that a RPC will have parameters.  End users need 
	 * a comment to understand what are the acceptable parameters. 
	 * @return a comment describing the code that creates a RPC
	 */
	public void genCreatorComment();
	
	/**
	 * makes code that handles the creation of a RPC 
	 * @return code that creates a RPC based on pre-defined parameters that are passed in. 
	 */
	public void genCreatorCode();
	
	/**
	 * Makes code that handles the parsing of a RPC
	 * @return code for a set of methods to parse a RPC. 
	 * Each parameter that is needed to create the RPC needs a 'undo' method 
	 * that and interpret an incoming RPC and return the value of a parameter 
	 */
	public void genParserCode();	
	
	/**
	 * generate code that, given a packet can determine if it is of that type of packet
	 * @return code
	 */
	public void genCheckPacketType();

}
