package CodeGen;


/**
 * Contains methods that generate snippets of code to work with parameters 
 * @author Galia
 *
 */
public interface IParamGen {

	
	/**
	 * There needs to be a way to get the value of a parameter in a packet
	 * @param name - name of the method this is for
	 * @param depthIntoPacket - how many bytes into the packet does this parameter occur
	 * @return a method that returns the value of the specified parameter
	 */
	void getGetter(String name, int depthIntoPacket);
	
	
	/**
	 * @return size in bytes of this parameter 
	 */
	int getLength();

	/**
	 * @return name of the parameter
	 */
	String getName();

	/**
	 * @return the data type of the parameter
	 * (this implementation is generation language specific) 
	 */
	String getType();

	/**
	 * @return the comment for this parameter. It might contain information about valid values
	 */
	String getComment();
	
	/**
	 * Return true if the parameter is a constant.
	 * Return false id the parameter is a variable.
	 */
	boolean isConstant();
	
	/**
	 * This is for code generation.  How should the value be printed?
	 * @param value the string value to convert to a printable java value
	 * @return a printable java value
	 */
	String valueAsByteArray();

}
