package javaCodeGen;

import java.util.Iterator;

import CodeGen.IMethodGen;
import MDL.*;

public class MethodJavaGen extends JavaElement implements IMethodGen{
	private Method method;
	public MethodJavaGen(Method next) {
		this.method = next;
	}
	/**
	 * I chose to implement this using a table method, where are are several
	 * constant values stored in a data structure in the class.
	 * This simply generates a comment letting programmers know what there are
	 * all of these private values.
	 */
	public void genCommentForConsts(){
		setI(1);
		addL("/*");
		addL(" * The Follow Set of Constants are for the " +method.getName() + " method.");
		addL(" */");
		setI();
	}

	/**
	 * Generates an array of byte arrays.  Are element in this array represents
	 * a parameter in the method
	 */
	public void genComponentsArray(){
		setI(1);
		addL("//Represents reach component of the packet");
		addL("private static byte[][] " + method.getName() + "_components = {");
		Iterator<Param> params = method.getParams();
		while(params.hasNext()){
			Param p = params.next();
			String line = new ParamJavaGen(p).valueAsByteArray() + ",";
			line +=  " //" + p.getName();
			if(p.getComment()!=null){
				line += ": " + p.getComment();
			}
			addL(line, 1);
		}
		addL("};");
		setI();
	}

	/**
	 * In my implementation the packet length needs to be calculated a lot.
	 * Since the length will always be the same, I pre-calculate it
	 */
	public void genPacketLen(){
		int totLen = 0;
		Iterator<Param> p= method.getParams();
		while(p.hasNext()){
			totLen += p.next().getLength();
		}		
		setI(1);
		addL("//the total length of the packet");
		addL("//note that this could be calculated at run time");
		addL("private static int putServo_len = " + totLen +";");
		setI();
	}

	/**
	 * This prints an array to see if something is a constant or a variable.  
	 * This is checked against when the CheckPacketType method is invoked
	 */
	public void genConstantArray(){
		Iterator<Param> p = method.getParams();
		String line = "";
		setI(1);
		addL("//indicates which of the values are constant");
		addL("//and which ones will be determined at run time");
		addL("private static boolean[] " + method.getName() + "_const = {");
		while(p.hasNext()){
			if(p.next().isConstant()){
				line +="true, ";
			}else{
				line+="false, ";
			}
		}
		addL(line, 1);
		addL("};");
		setI();
	}
	
	@Override
	public void genCreatorComment() {
		Iterator<Param> pi = method.getParams();
		setI(1);
		addL("/**");
		addL(" *creates a packet");
		if (method.hasComment()) {addL(" *" + method.getComment());};
		while(pi.hasNext()){
			Param p = pi.next();
			if(!p.isConstant()) {
				addL(" *@param " + p.getName() + " " +p.getComment());
			}
		}
		addL(" *@return array of bytes representing the data packet");
		addL(" */");
		setI();
	}
	 
	@Override
	public void genCreatorCode() {
		Iterator<Param> params = method.getParams();
		int numParams = 0;
		String line;
		setI(2);
		line = "public static byte[] "; 
		line += method.getName() +"(";
		while(params.hasNext()){
			ParamJavaGen param = new ParamJavaGen(params.next());
			if (!param.isConstant()){
				line += param.getType();
				line += " ";
				line += param.getName();
				line += ",";
			}
			numParams++;
		}
		line = (String) line.subSequence(0, line.lastIndexOf(',')); //remove extra ','
		line += ") throws Exception{";
		addL(line, -1);
		
		params = method.getParams();
		for(int i =0; i<numParams; i++){
			Param param = params.next();
			if(!param.isConstant()){
				addL("int i_" + param.getName() + " = " + i + ";");
				line = method.getName()+ "_components[i_" + param.getName()+"]";
				if(param.getLength() == 1 && 
						param.getType().equals(Param.Type.UnsignedInt) ||
						param.getType().equals(Param.Type.Integer)){
					addL(line +"[0] = (byte) " + param.getName() + ";");
				}else{
					addL(line + "= "+ param.getName() +";");
				}				
			}			
		}
		addL("byte[] result = Utils.merge(putServo_components, putServo_len);");			
		addL("return result;");			
		addL("}", -1);
		addL();	
		setI();
	}
	@Override
	public void genCheckPacketType() {
		setI(2);
		addL("/*", -1);
		addL(" * If all the constant values for a putServo match then, its this type of packet", -1);
		addL(" * If all the constant values for a putServo match then, its this type of packet", -1);
		addL(" * @param packet", -1);
		addL(" * @return true if all constant values match", -1);
		addL(" */",-1);
		addL("public static boolean is" + method.getName() +"Packet(byte[] packet){", -1);
		addL("//If these are not the same length, then they are not the same packet.");
		addL("boolean matches = (packet.length ==" + method.getName()+ "_len);");
		addL("int bytesSoFar = 0;");
		String line = "for(int component = 0; component < ";
		line += method.getName();
		line += "_components.length && matches; component++){";
		addL(line);
		line = "for(int bytesIn = 0; bytesIn < ";
		line += method.getName();
		line += "_components[component][bytesIn]; bytesIn++){";
		addL(line, 1);
		addL("if(putServo_const[component]){ //If it is a constant value...", 2);
		addL("//make sure that it matches.", 3);
		addL("matches = packet[bytesSoFar]==" + method.getName() + "_components[component][bytesIn];",3);
		addL("}", 2);
		addL("bytesSoFar++;",2);
		addL("}", 1);
		addL("}", 0);
		addL("return matches;",0);
		addL("}", -1);		
		setI();
	}
	@Override
	public void genParserCode() {
		Iterator <Param> params = method.getParams();
		int depthIntoPacket = 0;
		while(params.hasNext()){
			Param param = params.next();			
			if(!param.isConstant()){
				new ParamJavaGen(param).getGetter(method.getName(), depthIntoPacket);
			}
			depthIntoPacket += param.getLength();
		}		
	}	
}
